"""stock_keeping_units

Revision ID: 0670868fe171
Revises: 6fb6c96fd408
Create Date: 2021-09-25 12:17:58.540829

"""
import sqlalchemy as sa

from alembic import op
from sqlalchemy import Boolean, Numeric, Unicode, column, select, table
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
from sqlalchemy.dialects.postgresql import UUID


revision = "0670868fe171"
down_revision = "6fb6c96fd408"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "stock_keeping_units",
        sa.Column("id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("product_id", postgresql.UUID(as_uuid=True), nullable=False),
        sa.Column("is_default", sa.Boolean(), nullable=False),
        sa.Column("units", sa.Unicode(length=255), nullable=False),
        sa.Column("fraction", sa.Numeric(precision=15, scale=5), nullable=False),
        sa.Column("fraction_units", sa.Unicode(length=255), nullable=False),
        sa.Column("product_yield", sa.Numeric(precision=15, scale=5), nullable=False),
        sa.Column("cost_price", sa.Numeric(precision=15, scale=2), nullable=False),
        sa.Column("sale_price", sa.Numeric(precision=15, scale=2), nullable=False),
        sa.ForeignKeyConstraint(
            ["product_id"], ["products.id"], name=op.f("fk_stock_keeping_units_product_id_products")
        ),
        sa.PrimaryKeyConstraint("id", name=op.f("pk_stock_keeping_units")),
        sa.UniqueConstraint("product_id", "units", name=op.f("uq_stock_keeping_units_product_id")),
    )
    op.create_index(
        "unique_true_is_default",
        "stock_keeping_units",
        ["product_id"],
        unique=True,
        postgresql_where=sa.text("is_default = true"),
    )
    op.drop_constraint("uq_products_name", "products", type_="unique")
    op.create_unique_constraint(op.f("uq_products_name"), "products", ["name"])

    sku = table(
        "stock_keeping_units",
        column("id", UUID(as_uuid=True)),
        column("product_id", UUID(as_uuid=True)),
        column("is_default", Boolean),
        column("units", Unicode(255)),
        column("fraction", Numeric(precision=15, scale=5)),
        column("fraction_units", Unicode(255)),
        column("product_yield", Numeric(precision=15, scale=5)),
        column("cost_price", Numeric(precision=15, scale=2)),
        column("sale_price", Numeric(precision=15, scale=2)),
    )
    product = table(
        "products",
        column("id", UUID(as_uuid=True)),
        column("units", Unicode(255)),
        column("fraction", Numeric(precision=15, scale=5)),
        column("fraction_units", Unicode(255)),
        column("product_yield", Numeric(precision=15, scale=5)),
        column("cost_price", Numeric(precision=15, scale=2)),
        column("sale_price", Numeric(precision=15, scale=2)),
    )

    op.execute(
        sku.insert().from_select(
            [
                sku.c.id,
                sku.c.product_id,
                sku.c.is_default,
                sku.c.units,
                sku.c.fraction,
                sku.c.fraction_units,
                sku.c.product_yield,
                sku.c.cost_price,
                sku.c.sale_price,
            ],
            select(
                [
                    product.c.id,
                    product.c.id,
                    True,
                    product.c.units,
                    product.c.fraction,
                    product.c.fraction_units,
                    product.c.product_yield,
                    product.c.cost_price,
                    product.c.sale_price,
                ]
            ),
        )
    )

    op.drop_column("products", "cost_price")
    op.drop_column("products", "units")
    op.drop_column("products", "sale_price")
    op.drop_column("products", "product_yield")
    op.drop_column("products", "fraction_units")
    op.drop_column("products", "fraction")

    op.drop_constraint("uq_rate_contract_items_rate_contract_id", "rate_contract_items", type_="unique")
    op.drop_constraint("fk_rate_contract_items_product_id_products", "rate_contract_items", type_="foreignkey")
    op.alter_column("rate_contract_items", "product_id", new_column_name="sku_id")
    op.create_unique_constraint(
        op.f("uq_rate_contract_items_rate_contract_id"), "rate_contract_items", ["rate_contract_id", "sku_id"]
    )
    op.create_foreign_key(
        op.f("fk_rate_contract_items_sku_id_stock_keeping_units"),
        "rate_contract_items",
        "stock_keeping_units",
        ["sku_id"],
        ["id"],
    )

    op.drop_constraint("batches_ProductID_fkey", "batches", type_="foreignkey")
    op.alter_column("batches", "product_id", new_column_name="sku_id")
    op.create_foreign_key(
        op.f("fk_batches_sku_id_stock_keeping_units"), "batches", "stock_keeping_units", ["sku_id"], ["id"]
    )

    op.drop_constraint("inventories_ProductID_fkey", "inventories", type_="foreignkey")
    op.drop_column("inventories", "product_id")

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "products", sa.Column("fraction", sa.NUMERIC(precision=15, scale=5), autoincrement=False, nullable=False)
    )
    op.add_column("products", sa.Column("fraction_units", sa.VARCHAR(length=255), autoincrement=False, nullable=False))
    op.add_column(
        "products", sa.Column("product_yield", sa.NUMERIC(precision=15, scale=5), autoincrement=False, nullable=False)
    )
    op.add_column(
        "products", sa.Column("sale_price", sa.NUMERIC(precision=15, scale=2), autoincrement=False, nullable=False)
    )
    op.add_column("products", sa.Column("units", sa.VARCHAR(length=255), autoincrement=False, nullable=False))
    op.add_column(
        "products", sa.Column("cost_price", sa.NUMERIC(precision=15, scale=2), autoincrement=False, nullable=False)
    )
    op.drop_constraint(op.f("uq_products_name"), "products", type_="unique")
    op.create_unique_constraint("uq_products_name", "products", ["name", "units"])
    op.drop_index(
        "unique_true_is_default", table_name="stock_keeping_units", postgresql_where=sa.text("is_default = true")
    )
    op.drop_table("stock_keeping_units")
    # ### end Alembic commands ###
