import uuid

from datetime import datetime
from typing import Optional

from pydantic.main import BaseModel

from . import to_camel


class ClientIn(BaseModel):
    name: str
    enabled: bool
    otp: Optional[int]


class Client(ClientIn):
    id_: uuid.UUID
    code: int
    creation_date: datetime

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel


class ClientList(Client):
    last_user: str
    last_date: Optional[datetime]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {datetime: lambda v: v.strftime("%d-%b-%Y %H:%M")}
