import uuid

from datetime import date, datetime
from typing import List, Optional, Union

from pydantic import BaseModel, validator

from . import to_camel


class LockDate(BaseModel):
    days: Optional[int]
    date_: Optional[date]

    class Config:
        fields = {"date_": "date"}
        json_encoders = {
            date: lambda v: v.strftime("%d-%b-%Y"),
        }

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[None, date, str]) -> Optional[date]:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()


class VoucherTypesSelected(BaseModel):
    id_: int
    name: str

    class Config:
        fields = {"id_": "id"}


class AccountTypesSelected(BaseModel):
    id_: int
    name: str

    class Config:
        alias_generator = to_camel


class LockInformation(BaseModel):
    id_: Optional[uuid.UUID]

    valid_from: Optional[date]
    valid_till: Optional[date]

    voucher_types: List[VoucherTypesSelected]
    account_types: List[AccountTypesSelected]

    start: LockDate
    finish: LockDate

    index: int

    class Config:
        alias_generator = to_camel
        json_encoders = {
            date: lambda v: v.strftime("%d-%b-%Y"),
        }

    @validator("valid_from", pre=True)
    def parse_valid_from(cls, value: Union[None, date, str]) -> Optional[date]:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("valid_till", pre=True)
    def parse_valid_till(cls, value: Union[None, date, str]) -> Optional[date]:
        if value is None:
            return None
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()


class Maintenance(BaseModel):
    enabled: bool
    user: str


class AttendanceCount(BaseModel):
    attendance_count: int

    class Config:
        alias_generator = to_camel
