import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { debounceTime, distinctUntilChanged } from 'rxjs/operators';

import { Account } from '../../core/account';
import { AccountType } from '../../core/account-type';

import { AccountListDataSource } from './account-list-datasource';

@Component({
  selector: 'app-account-list',
  templateUrl: './account-list.component.html',
  styleUrls: ['./account-list.component.css'],
})
export class AccountListComponent implements OnInit, AfterViewInit {
  @ViewChild('filterElement', { static: true }) filterElement?: ElementRef;
  @ViewChild(MatPaginator, { static: true }) paginator?: MatPaginator;
  @ViewChild(MatSort, { static: true }) sort?: MatSort;
  dataSource: AccountListDataSource;
  filter: Observable<string>;
  form: FormGroup<{
    filter: FormControl<string>;
  }>;

  list: Account[] = [];
  accountTypes: AccountType[] = [];

  /** Columns displayed in the table. Columns IDs can be added, removed, or reordered. */
  displayedColumns = ['name', 'type', 'isActive', 'isReconcilable', 'costCentre'];

  constructor(private route: ActivatedRoute) {
    this.form = new FormGroup({
      filter: new FormControl<string>('', { nonNullable: true }),
    });
    // Listen to Filter Change
    this.filter = this.form.controls.filter.valueChanges.pipe(
      debounceTime(150),
      distinctUntilChanged(),
    );

    this.dataSource = new AccountListDataSource(this.list, this.filter);
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { list: Account[]; accountTypes: AccountType[] };
      this.accountTypes = data.accountTypes;
      data.list.forEach(
        (x) =>
          (x.type = new AccountType({
            name: this.accountTypes.find((y) => y.id === x.type)?.name,
          })),
      );
      this.list = data.list;
    });
    this.dataSource = new AccountListDataSource(this.list, this.filter, this.paginator, this.sort);
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.filterElement) {
        this.filterElement.nativeElement.focus();
      }
    }, 0);
  }
}
