import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

const serviceName = 'EmployeeFunctionsService';

@Injectable({ providedIn: 'root' })
export class EmployeeFunctionsService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  creditSalary(month: string): Observable<boolean> {
    const url = '/api/credit-salary';
    return this.http
      .post<boolean>(url, { month })
      .pipe(catchError(this.log.handleError(serviceName, 'creditSalary'))) as Observable<boolean>;
  }

  uploadFingerprints(fingerprintFile: File): Observable<boolean> {
    const url = '/api/fingerprint';
    const fd = new FormData();
    fd.append('fingerprints', fingerprintFile);
    return this.http
      .post<boolean>(url, fd)
      .pipe(
        catchError(this.log.handleError(serviceName, 'uploadFingerprints')),
      ) as Observable<boolean>;
  }
}
