import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { Report } from './report';

const url = '/api/entries';
const serviceName = 'EntriesService';

@Injectable({
  providedIn: 'root',
})
export class EntriesService {
  constructor(private http: HttpClient, private log: ErrorLoggerService) {}

  list(
    start_date: string | null,
    finish_date: string | null,
    posted: boolean | null,
    page_size: number,
    page_index: number,
    sort: string,
    direction: string,
    issue: boolean,
  ): Observable<Report> {
    let params = new HttpParams();
    if (start_date !== null) params = params.set('s', start_date);
    if (finish_date !== null) params = params.set('f', finish_date);
    if (posted !== null) params = params.set('p', posted.toString());
    params = params
      .set('ps', page_size)
      .set('pi', page_index)
      .set('a', sort)
      .set('d', direction)
      .set('i', issue.toString());

    return this.http
      .get<Report>(url, { params })
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Report>;
  }
}
